@@ -62,7 +62,7 @@ do_time_out_success <- function(data, day_num, game_code, team_sel, verbose) {
   # Case when the time out is at the last row of the data, for example Joventut in 8/103315:
   lr <- which(to_pos_aux == nrow(data))
   if (length(lr) != 0) {
-    to_pos_aux <- to_pos_aux[-which(to_pos_aux == nrow(data))]
+    to_pos_aux <- to_pos_aux[-lr]
   }
   
   # Case of no time out requested, for example Unicaja in 6/103297:
@@ -103,8 +103,7 @@ do_time_out_success <- function(data, day_num, game_code, team_sel, verbose) {
         if (data$period[j] == data$period[j - 1]) {
           if (data$team[j] == team_sel) {
             to_pos_after <- c(to_pos_after, j)
-          }else if (data$team[j] != team_sel & grepl("Falta Personal|T\u00e9cnica", 
-                                                     data$action[j])) {
+          }else if (data$team[j] != team_sel & grepl("Falta Personal|T\u00e9cnica", data$action[j])) {
             to_pos_after <- c(to_pos_after, j)
           }else{
             break()
@@ -118,7 +117,7 @@ do_time_out_success <- function(data, day_num, game_code, team_sel, verbose) {
       to_pos_after_points_nona <- to_pos_after_points[!is.na(to_pos_after_points)]
       
       if (length(to_pos_after_points_nona) != 0) {
-        # sum() just in case there were more than action with points, such as foul plus free throw.
+        # sum() just in case there were more than one action with points, such as foul plus free throw.
         points_scored <- points_scored + sum(to_pos_after_points_nona)
         times_succ <- times_succ + 1
       }
